package com.sf.vertx.handle;

import java.util.concurrent.TimeUnit;

import org.springframework.data.redis.core.RedisTemplate;

import com.sf.vertx.api.pojo.ApiCurrentLimitingConfig;
import com.sf.vertx.api.pojo.AppCurrentLimitingConfig;
import com.sf.vertx.constans.RedisKeyConfig;
import com.sf.vertx.init.DynamicBuildServer;
import com.sf.vertx.service.impl.AppConfigServiceImpl;
import com.sf.vertx.utils.SpringUtils;

import cn.hutool.core.thread.ThreadUtil;
import io.vertx.ext.web.RoutingContext;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class RedisRateLimiter {
	public Boolean acquire(RoutingContext rc, int pattern) {
		String appCode = rc.request().headers().get(DynamicBuildServer.SAC_APP_HEADER_KEY);
		// TODO 先测试app模式,后面通过app缓存获取模式
		String key = null;
		switch (pattern) {
		case 1:
			AppCurrentLimitingConfig appCurrentLimitingConfig = AppConfigServiceImpl.getAppCurrentLimitingConfig(appCode);
			key = RedisKeyConfig.APP_CURRENT_LIMITING_CONFIG_KEY + ":" + appCode;
			return rateLimiter(key, appCode, appCurrentLimitingConfig.getThreshold(), appCurrentLimitingConfig.getTimeWindow());
		case 2:
			ApiCurrentLimitingConfig apiCurrentLimitingConfig = AppConfigServiceImpl.getApiCurrentLimitingConfig(appCode);
			key = RedisKeyConfig.APP_CURRENT_LIMITING_CONFIG_KEY + ":" + appCode + ":" + rc.request().uri() + ":"
					+ rc.request().method();
			return rateLimiter(key, appCode, apiCurrentLimitingConfig.getThreshold(), apiCurrentLimitingConfig.getTimeWindow());
		default:
			break;
		}
		return false;

	}

	@SuppressWarnings("unchecked")
	private Boolean rateLimiter(String key, String appCode, Integer threshold, Integer timeWindow) {
		RedisTemplate<String, Integer> redisTemplate = SpringUtils.getBean("redisTemplate", RedisTemplate.class);
		Object count = redisTemplate.opsForValue().get(key);
		// redisTemplate.delete(key);
		log.info("count:{}", count);
		if (count == null) {
			// 初始化,设置过期时间
			ThreadUtil.execAsync(() -> {
				add(timeWindow, redisTemplate, key);
			});
		} else if (Integer.valueOf(count.toString()) < threshold) {
			ThreadUtil.execAsync(() -> {
				increment(redisTemplate, key);
			});
		} else {
			return false;
		}
		return true;
	}

	private void add(Integer timeWindow, RedisTemplate<String, Integer> redisTemplate,
			String key) {
		redisTemplate.opsForValue().increment(key);
		log.info("redis app threshold: {}", redisTemplate.opsForValue().get(key));
		redisTemplate.expire(key, timeWindow, TimeUnit.SECONDS);
	}

	private void increment(RedisTemplate<String, Integer> redisTemplate, String key) {
		redisTemplate.opsForValue().increment(key);
	}
}
